#!/usr/bin/env python

"""
Stuff for working with the specific setup for the TTN136B Nov 2001.
Cruise to the Humboldt Slide/Eel River area.  Chief Scientist was Kurt
Schwehr.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: ttn136b.py,v 1.8 2005/11/09 19:05:03 schwehr Exp $
"""

# FIX: is this the right naming convention for these variables and how
# are they used?
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/409413

__author__ = "Kurt Schwehr"
__revision__ = "$Id: ttn136b.py,v 1.8 2005/11/09 19:05:03 schwehr Exp $"
__version__ = "0.1"
# __all__ ???

import unittest
import sqlite # for DepthLookup

dbFileName = "/Users/schwehr/projects/xcore/data/TTN136B/ttn136b.db"

class SampleName:
    """ Parse a Kurt style pmag sample cube name """
    def __init__(self,nameString=""):
        if 0==len(nameString):
            # Make a blank object
            self.cruise="None"
            self.coreNum=-1
        else:
            #self.cruise,core,section,self.depthOffset=nameString.split("-")
            lst = nameString.split("-")

            self.cruise=lst[0]

            if 3==len(lst):
                # No section
                self.coreNum=int(lst[1][0:2])
                self.coreType=lst[1][2]
                self.coreHalf=lst[1][3]
                if self.coreHalf == 'C' or self.coreHalf == 'c':
                    # Found a 01pc style name (not 01pw)
                    self.coreHalf = 'w'
                self.section = None
                self.depthOffset=float(lst[2])
                #print "depth = ",self.depthOffset
            elif 4 == len(lst):
                # has section.  e.g. ttn136b-05pc-s5-020
                try:
                    self.coreNum=int(lst[1][0:2])
                    self.coreType=lst[1][2]
                    self.coreHalf=lst[1][3]
                except:
                    # If sample name is missing the 0 from the core... ttn136b-5pw-s5-020 
                    self.coreNum=int(lst[1][0:1])
                    self.coreType=lst[1][1]
                    self.coreHalf=lst[1][2]
                #print "coret", self.coreType
                #print "coreh", self.coreHalf
                if self.coreHalf == 'C' or self.coreHalf == 'c':
                    # Found a 01pc style name
                    self.coreHalf = 'w'
                # list[2] == 's2'
                self.section=int(lst[2][1])

                
                self.depthOffset=float(lst[3])
                #print "depth = ",self.depthOffset
            else:
                print nameString, "samplename len = ", len(lst)
                assert(False)
                self.section=int(lst[2][1])
                self.depthOffset=float(lst[3])
                
            # FIX: check for optional part
#            if 4==len(lst):
#                # There was no L,C,R tag for left, center, right
#                self.horizPart=None
#            else:
#                self.horizPart=lst[4]

    def __str__(self):
        """ Reconstruct the original sample name string """
        tmpStr=self.cruise+"-"+("%d"%self.coreNum)+self.coreType+self.coreHalf
        tmpStr+="-s" + ("%d" % self.section) + "-"
        o = self.depthOffset
        if ( (round(o)-0.01<o) & (round(o)+0.01>o) ):
            tmpStr+="%03d" % int(o)
        else:
            tmpStr+="%03.1f" % o
        if (self.horizPart):
            tmpStr+="-"+self.horizPart
        else:
            pass # Then just drop it
        return tmpStr



class DepthLookup:
    def __init__(self, db_cx):
        """ Lookup the section table and make a dictionary.  Loads from the database section table """
        cu = db_cx.cursor()
        cu.execute ("SELECT cruise,corenum,coretype,section,sectopdepth FROM sections")
        self.table = {}
        for row in cu.fetchall():
            key = row.cruise + "-" + str(row.corenum) + "-" + str(row.section) + "-" + row.coretype
            #print "key=",key
            self.table[key] = row.sectopdepth

    def lookup(self, sampleNameStr):
        """
        Return the depth in cm
        
        Use this lookup to go faster than constant database access to get depths
        """
        sn = SampleName(sampleNameStr)
        if None == sn.section or 0 == sn.section:
            #print "No section for ",sampleNameStr, "  Using depthOffset of ", sn.depthOffset
            return sn.depthOffset
        key = sn.cruise + "-" + str(sn.coreNum) + '-' + str(sn.section) + '-' + sn.coreType
        depth = self.table[key] + sn.depthOffset
        #print "Found: ",sampleNameStr,key, depth, "(",self.table[key],"+",sn.depthOffset,")"
        return depth

    def lookup2(self,corenum,coretype,section,relativedepth):
        """
        corenum       - 1..7
        coretype      - t for trigger, p for piston
        section       - section of the core.  1 is the upper most section (close to bottom-water interface)
        relativedepth - depth in cm in that section
        lookup2(2,'t','1',13)
        """
        key = 'ttn136b-' +str(corenum)+'-'+str(section)+'-'+coretype
        return self.table[key]+relativedepth

def getDepth(db_cx, samplename, depthTable='ams'):
    """
    Lookup the depth of a sample from a table.

    PARAMETERS:
    db_cx      - sqlite database connection
    samplename - find the depth for this sample
    depthTable - the table that has both samplename and depth fields

    RETURNS:
    depth in cm
    """
    cu = db_cx.cursor()
    cu.execute("SELECT depth FROM "+depthTable+" WHERE samplename='"+samplename+"';")
    row = cu.fetchone()
    depth = row['depth']
    return depth


class TTN136BTests(unittest.TestCase):
    """
    Unit Tests to make sure all is well

    Run the test cases by doing:

       ./ttn136b.py
    """
    def test_SampleName1(self):
        """Test the sample name parsing - no section number - abs depth"""
        sn1 = SampleName('ttn136b-01pw-002')
        self.failUnless('ttn136b'==sn1.cruise)
        self.failUnless(1==sn1.coreNum)
        self.failUnless('p'==sn1.coreType)
        self.failUnless('w'==sn1.coreHalf)
        self.failUnless(None==sn1.section)
        self.failUnless(2==sn1.depthOffset)


# Test code
if __name__ == '__main__':
    unittest.main()
