#!/usr/bin/env python

"""
Magnetic and paleomagnetic unit conversions and constants

SEE ALSO: Lisa Tauxe's 2005 Paleomagnetic Lectures, Ch 1, P 9

http://earthref.org/MAGIC/books/Tauxe/2005/

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: units.py,v 1.8 2005/10/30 03:46:14 schwehr Exp $
"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.8 $'.split()[1] # See man ident
__date__ = '$Date: 2005/10/30 03:46:14 $'.split()[1]
__author__ = 'Kurt Schwehr'

from math import pi
from pmag_verbosity import BOMBASTIC,VERBOSE,TRACE,TERSE,ALWAYS


######################################################################
# CONSTANTS
######################################################################

mu0_si = u0_si = 4 * pi / 1e-7 # H/m
mu0_cgs = u0_cgs = 1

######################################################################
# OTHER CONVERSIONS
######################################################################

def deg2rad(degrees):
    return (degrees / 180.) * pi
def rad2deg(radians):
    return (radians / pi) * 180.


######################################################################
# MAGNETIC CONVERSIONS
######################################################################

# 10^4 Oe = 1 T

def Oe2T(Oe):
    "Convert Oersteds to Teslas... FIX: reference?"
    T = Oe * 1e-4
    return 

def T2Oe(T):
    "Convert  Teslas to Oersteds... FIX: reference?"
    Oe = T * 1e4
    return 

# FIX: what to name these functions?!?!?
# Probably should have a couple names for the same thing?

def H_si2cgs(si):
    """Magnetic field (H) from A/m to Oersted (oe)"""
    cgs = si / ( 4 * pi ) * 1000
    return cgs

def H_cgs2si(cgs):
    """Magnetic field (H) from Oersted (oe) to A/m"""
    si = cgs * 4 * pi / 1000.
    return si

def m_si2cgs(si):
    'Magnetic moment (m) from Am^2 to emu'
    cgs = si * 1000
    return cgs

def m_cgs2si(cgs):
    'Magnetic moment (m) from emu to Am^2'
    si = cgs / 1000.
    return si

def B_cgs2si(cgs):
    'Magnetic Induction (B) from Gauss to T'
    si = cgs * 1e-4
    return si

def B_si2cgs(si):
    ' Magnetic Induction (B) from T to Gauss'
    cgs = si * 1e4
    return cgs

import unittest
class testconversions(unittest.TestCase):
    """
    FIX: would be good to have some better tests with know conversions!
    """
    def test_H(self):
        """Check H_si2cgs and H_cgs2si"""
        # Simple identities...
        self.failUnless(0==H_si2cgs(0))
        self.failUnless(0==H_cgs2si(0))
        self.failUnless(1==H_si2cgs(H_cgs2si(1)))
        #self.failUnlessAlmostEqual
        # FIX: More interesting tests...

    def test_m(self):
        """Check m_si2cgs and m_cgs2si"""
        # Simple identities...
        self.failUnless(0==m_si2cgs(0))
        self.failUnless(0==m_cgs2si(0))
        self.failUnless(1==m_si2cgs(m_cgs2si(1)))
        # FIX: More interesting tests...

    def test_B(self):
        """Check B_si2cgs and B_cgs2si"""
        # Simple identities...
        self.failUnless(0==B_si2cgs(0))
        self.failUnless(0==B_cgs2si(0))
        self.failUnless(1==B_si2cgs(B_cgs2si(1)))
        # FIX: More interesting tests...
        
######################################################################
    def test_Hyst1(self):
        """ Hysteresis slope to high field susceptability
        """
        emuPerOe = 1e-8
        slopeSI = 1e-7
        siVolChi = 8.18e-5
        self.failUnlessAlmostEqual(slopeSI,emuPerOe_2_SIslope(emuPerOe))

        self.failUnlessAlmostEqual(siVolChi,emuPerOe_2_volChiSI(emuPerOe, massGrams = 2.2e-3, densityKg_m3=1500),places=4)

        self.failUnlessAlmostEqual(siVolChi,agfmS_2_volChiHiSI(slopeAm2perT=slopeSI,massKg=2.2e-6, densityKg_m3=1500),places=3)
            
def emuPerOe_2_SIslope(emuPerOe):
    """
    Convert the CGS slope of emu/Oe (e.g. from 2900 AGFM) to SI slope (Am^2 / T)

    Hysteresis
    """
    Am2PerOe = m_cgs2si(emuPerOe)  # * 1e-3
    Am2PerT = Am2PerOe / 1e-4  # Convert the bottom from Oe to T
    
    return Am2PerT

def emuPerOe_2_volChiSI(emuPerOe, massGrams, densityKg_m3 = 1500):
    """ Convert slope from AGFM in emu/Oe, mass, and density to a volume normalized High Field Susceptability (in SI)

    emuPerOe - slope from the Model 2900 AGFM (cgs units)
    massGrams - weight of the sample
    densityKg_m3 - density in kg/m^3

    returns - volume normalized 
    """
    assert (massGrams < 1e-1) # Samples on the agfm are small
    assert (densityKg_m3 > 500)
    assert (densityKg_m3 < 99999) # Wow that would be dense
    emuPerOeGm = emuPerOe/massGrams
    AperMperOe = emuPerOeGm * densityKg_m3
    AperMperT = AperMperOe * 1e4 # Convert Oe to T on the bottom
    SI = AperMperT / 8.33e5  # T to A/m on the bottom
    return SI

def agfmS_2_volChiHiSI(slopeAm2perT,massKg, densityKg_m3=1500):
    """SI slope from AGFM to SI susceptibility normalized by volume

    A/(m*T) to SI volume normalized
    """
    
    SI = (slopeAm2perT/massKg) * densityKg_m3 / mu0_si
    return SI


# FIX: What is the best way to do unit test?
# Opt parse with a --unit-test option?
if __name__ == '__main__':
    from pmag_kds_version import VERSION

    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options] hyst1.raw hyst2.raw hyst3.raw ...",
                            version="%prog "+VERSION+' - '+__version__)
    myparser.add_option('--test','--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')

    # FIX: add command line options to do conversions

    import pmag_verbosity
    pmag_verbosity.addVerbosityOptions(myparser)

    import sys
    (options,args) = myparser.parse_args()

    if options.unittest:
        sys.argv = [sys.argv[0]] # unit test does not like extra argv's!
        unittest.main()
