#!/usr/bin/env python

"""
Utilities for generating OpenInventor ASCII files

GenIV    -- base class that provides low level nodes and objects
GenXCore -- derived class for geologic cores


FIX: add a flag to emit html highlighted code!


$Id: xcore.py,v 1.24 2005/10/29 15:31:07 schwehr Exp $

COPYRIGHT:

    Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""



import os	# os.remove()
import sys	# sys.exit()
import string
import platform # system information like mac_ver()

import datetime 	# for date.today()

# Use X11rgb instead for general lookup of colors
black = [0,0,0]
red   = [1,0,0]
green = [0,1,0]
blue  = [0,0,1]
white = [1,1,1]
#import Image	# pil-py --- python image library

__author__ = "Kurt Schwehr"
__revision__ = "$Id: xcore.py,v 1.24 2005/10/29 15:31:07 schwehr Exp $"
__license__ = "GPL"

######################################################################
class X11rgb:
    def __init__(self,rgbfile="/usr/X11R6/lib/X11/rgb.txt",maxvalue = 255):
        """
        Load in the rgb.txt file from X11.  Assumes X11 is available

        maxvalue -- what do you want the range to be.  Defaults to [0..255]
                    OpenInventor and others want [0..1] (maxvalue=1)

        FIX: if the table is not available, give them at least 8 colors or something
        """
        self.rgb={}  # Dictionary of (r,g,b) tuples

        if not os.access(rgbfile,os.R_OK):
            print "WARNING: X11 rgb file not avaible.  Tried ", filename
            print "giving you a minimal color palette"

            rgb['black'] = (0*maxvalue,0*maxvalue,0*maxvalue)
            rgb['red']   = (1*maxvalue,0*maxvalue,0*maxvalue)
            rgb['green'] = (0*maxvalue,1*maxvalue,0*maxvalue)
            rgb['blue']  = (0*maxvalue,0*maxvalue,1*maxvalue)
            rgb['white'] = (1*maxvalue,1*maxvalue,1*maxvalue)

            rgb['yellow'] = (1*maxvalue,1*maxvalue,0*maxvalue)
            rgb['magenta'] = (1*maxvalue,0*maxvalue,1*maxvalue)
            rgb['cyan'] = (0*maxvalue,1*maxvalue,1*maxvalue)

            rgb['orange'] = (1*maxvalue,.5*maxvalue,0*maxvalue)

        for line in open(rgbfile):
            if '!' == line[0]: continue  # Skip comments
            list = line.split()
            if 4 == len(list):
                colorName=list[3]
            else:
                # Cope with names that have spaces
                l2 = line.split('\t')
                colorName=l2[2]
                if 3!=len(l2):
                    print "WARNING: ",len(l2),colorName
                    continue
            
            r = list[0]
            g = list[1]
            b = list[2]
            if 255!=maxvalue:
                r = (float(r) / 255.) * maxvalue
                g = (float(g) / 255.) * maxvalue
                b = (float(b) / 255.) * maxvalue
            self.rgb[colorName] = (r,g,b)
        return



######################################################################
import math
def deg2rad(degrees):
    return (degrees / 180.) * math.pi
def rad2deg(radians):
    return (radians / math.pi) * 180.
######################################################################
        
class GenIV:
    """

    Class that handles generating OpenInventor 2.1/Coin ASCII geometry
    files.  The class tracks indenation for you and provides helpers
    for both low level and high level constructs.

    Most methods for this class emit IV geometry into the open file
    handle.
    """
    def __init__(self,filename="tmp.iv"):
        """Open a file handle for ASCII output to go into and write an IV header

        Initialize the indent level and indent string

        Arguments

        filename -- file open for writing
        """
        self.ivFile = open(filename,"w") 
        self.indentChar='  ' # Could also set this to '\t'
        self.indentLevel=0

    def header(self):
        """
        Write an OpenInventor 2.1 ASCII header
        """
        self.write("#Inventor V2.1 ascii\n")
        self.write("# Created by: "+__file__)
        self.write("# Author:     "+__author__)
        self.write("# Revision:   "+__revision__)
        self.write("# Date:       "+str(datetime.date.today()))
        self.write("# PWD:        "+os.getcwd())
        # FIX: switch this all to platform?
        self.write("# os:         "+str(os.uname()[0]))
        self.write("# host:       "+str(os.uname()[1]))
        self.write("# osver:      "+str(os.uname()[2]))
        self.write("# kernel:     "+str(os.uname()[3]))
        self.write("# arch:       "+str(os.uname()[4]))
        self.write("# OSX ver:    "+platform.mac_ver()[0])
        self.write("")

    def indent(self,numlevels=1):
        """Increase the indenting level.  Starts at zero."""
        self.indentLevel += numlevels

    def unindent(self,numlevels=1):
        """Decrease the indenting level.  Will not go below 0."""

        self.indentLevel -= numlevels
        if numlevels < 0: numlevels = 0

    def write(self,string="\n",newline=True):
        """
        Write a string to the open IV file.  Appends a newline unless
        you tell it not to do that.

        Arguments:
        string -  A string to put in the IV file
        newline - Set to false to prevent emitting an newline
        """

        pad=""
        for i in range(self.indentLevel): pad += self.indentChar
        self.ivFile.write(pad)
        if newline: outString = string + "\n"
        else: outString = string
        self.ivFile.write(outString)

    def comment(self,string="#"):
        """Add a comment string to file"""
        self.write("# "+string)

    def beginSeparator(self,comment=None):
        """Begin a separator block and increase indentation

        Arguments:
        comment - string to indentify a separator block
        FIX: add the ability to name an object
        """
        
        if comment:
            self.write("Separator { ",newline=False)
            self.comment(comment)
        else:
            self.write("Separator {")
        self.indent()


    def endSeparator(self,comment=None):
        """End the last separator and unindent
        
        FIX: use the name from the separator... maybe use a stack of names?
        """

        self.unindent()
        if comment:
            self.write("} ",newline=False)
            self.comment("End Separator: "+comment)
        else:
            self.write("} # End Separator")


    def baseColor(self, color):
        """
        Write a basecolor node

        Arguments

        color - [r,g,b] list or tuple - red, green, blue must be in the range of [0..1]
        """
        # FIX: check range for colors to be in [0..1]
        s=" "
        r=0; g=1; b=2
        self.write("BaseColor {rgb ["+str(color[r])+s+str(color[g])+s+str(color[b])+"]}")

    def includeFile(self,filename):
         """
         Have the current OpenInventor file include another file full of geometry

         Arguments

         out - open file handle to write to
         filename - string containing the file to be included by reference

         Wraps a separator for safety.
         """
         self.beginSeparator()
         self.write("File { name \""+filename+"\" }")
         self.endSeparator()
         return
    
    def translation(self,x,y,z):
        """Insert a translation node"""
        self.write("Translation { translation "+str(x)+" "+str(y)+" "+str(z)+" }")
        return
    
    def scale(self,x,y,z):
        """Insert a scale node"""
        self.write("Scale { scaleFactor "+str(x)+" "+str(y)+" "+str(z)+" }")
        return

    def rotationXYZ(self,axis="X",radians=None,degrees=None):
        """Rotate about X,Y, or Z"""
        axis = axis.upper()
        assert (1==len(axis))
        if radians: angle = radians; assert (not degrees)
        elif degrees: angle = deg2rad(degrees)
        self.write("RotationXYZ { axis "+str(axis)+" angle "+str(angle)+" }")
        return

    def text3(self,string):
        """ Write out a 3D text item """
        self.write("Text3{string[\""+str(string)+"\"]}\n")
        return

    def coordinate3(self,points):
        """Write an SoCoordinate3 node given a list of xyz's"""
        self.write("Coordinate3 { point [ ")
        # FIX: write out the fewest number of decimal points reasonable
        self.indent()
        for p in points:
            self.write(str(p[0])+" "+str(p[1])+" "+str(p[2])+",")
        self.unindent()
        self.write("] } # Coord3 ")

    def indexedLineSet(self,indices):
        """SoIndexedLineSet
        FIX: give example!
        """

        self.write("IndexedLineSet { coordIndex [")
        self.indent()
        #print indices
        for i in indices:
            #print i
            self.write(str(i)+', ',newline=False)
            if -1 == i:
                self.write("")
                #print "newline"
                       
        self.unindent()
        self.write("]} # IndexedLineSet")

    def indexedFaceSet(self,indices):
        """SoIndexedLineSet
        FIX: give example!
        """
        self.write("IndexedFaceSet { coordIndex [")
        self.indent()
        #print indices
        for i in indices:
            #print i
            self.write(str(i)+', ',newline=False)
            if -1 == i:
                self.write("")
                #print "newline"
                       
        self.unindent()
        self.write("]} # IndexedFaceSet")

    def lineSet(self,numVerticeList=None):
        """ Handle the LineSet node

        vertices = [ 3, 4, 2 ]
        """
        self.write("LineSet {")
        self.indent()
        self.write("numVertices [",newline=False)
        if not numVerticeList: self.write(" -1 ]")
        else:
            for n in numVerticeList:
                self.write(str(n)+",", newline=False)
            self.write("]")
        self.unindent()
        self.write("} # LineSet")
        
    def simpleLine(self,points):
        """
        Create a single line connecting the points
        """
        self.beginSeparator("simpleLine")
        self.coordinate3(points)
        self.lineSet()
        self.endSeparator("simpleLine")

    def pointSet(self,points,size=3,colors=None):
        """
        Create a pointset
        """
        self.beginSeparator("PointSet")
        if colors:
            # FIX: encapsulate materials
            self.write("Material { diffuseColor [")
            self.indent()
            for c in colors:
                self.write(str(c[0])+" "+str(c[0])+" "+str(c[0])+",",newline=False)
            self.write()
            self.unindent()
            self.write("] } #Material")
            self.write("MaterialBinding { value PER_PART }")
        self.coordinate3(points)
        self.write("DrawStyle { pointSize "+str(size)+" }")
        self.write("PointSet {}")
        self.endSeparator("PointSet")

######################################################################
class GenXCore(GenIV):
    """
    Create XCore Open Inventor ascii geometry files
    """
    def __init__(self,filename="tmp.iv"):
        """Open a file handle for ASCII output to go into and write an IV header"""
        GenIV.__init__(self,filename)
        self.header()
        
    def markRegions(self,layerList,width=10, color=red):
        """

        layerlist - list of range tuples to be marked
            e.g. [(74,76),(82,84),(104,105), (120,120.5) ]
        width - width of the marks in IV space
        color - [1,1,1]  list of rgb going from 0..1 for each
        """
        self.beginSeparator("markRegions")

        self.baseColor(color)

        # FIX: make this be one big coordinate3 and one big indexed face set
        for layer in layerList:
            top = str(layer[0])
            bot = str(layer[1])
            self.write("Coordinate3 { point [ -"+str(width/2)+" 0 -"+str(top)+",     "+str(width/2)+" 0.0 -"+str(top)+",")
            self.write("                      -"+str(width/2)+" 0 -"+str(bot)+",     "+str(width/2)+" 0.0 -"+str(bot)+" ] }")
            self.write("")
            self.write("IndexedFaceSet { coordIndex [ 2, 1, 0, -1,    2, 3, 1, -1 ]}")
        self.endSeparator("markRegions")

    def markClassifications(self,layerClassificationList,typeLUT,width=10):
        """
        layerlist - list of range tuples to be marked
            e.g. [(74,76,l'),(82,84,'vl'),(104,105,'b'), (120,120.5,'l') ]
        width - width of the marks in IV space
        """
        colors=X11rgb(maxvalue=1.0)

        self.beginSeparator("markRegions")

        for layer in layerClassificationList:
            #print layer
            top = layer[0]
            bot = layer[1]
            id = layer[2]  # string designator for a layer.  1 or 2 letters
            colorName = typeLUT[id][1]
            #print id,colorName
            self.baseColor(colors.rgb[colorName])
            points = [(-width/2., 0, -top),(width/2.,0,-top),\
                      (-width/2., 0, -bot),(width/2.,0,-bot)]
            self.coordinate3(points)
            self.indexedFaceSet([ 2, 1, 0, -1,    2, 3, 1, -1])
        self.endSeparator("markRegions")

    def sampleLocations(self,depths,minExtent=0.0,maxExtent=1.0,width = 5):
        """
        Make a graph showing the coverage of samples
        """
        colors=X11rgb(maxvalue=1.0)
        self.beginSeparator("sample locations");
        count = 0
        for depth in depths:
            top = depth+minExtent
            bot = depth+maxExtent
            if 0==count%2: colorName="gray95"
            else: colorName="gray45"

            self.baseColor(colors.rgb[colorName])
            # FIX: use FaceSet for smaller files
            points = [(0, 0, -top),(width,0,-top),\
                      (0, 0, -bot),(width,0,-bot)]
            self.coordinate3(points)
            self.indexedFaceSet([ 2, 1, 0, -1,    2, 3, 1, -1])
            count += 1
        self.endSeparator("sample locations");


    def corePhoto(self,imageFileName, widthInCM, heightInCM):
        """
        Writes out a separator block to wrap an image.  Takes an inventor
        file header in front just in case you want only this in the file.

        Arguments:

        outFile - open file handle to write header to
        imageFileName - valid image to load in open inventor.
            SGI Inventor can not handle a very wide range
        widthInCM   - assumes that one unit is one cm
        heightInCM  - assumes that one unit is one cm
        """

        self.write("# Starting CorePhoto")
        #self.write("Separator {")
        self.beginSeparator("corePhoto "+imageFileName)

        # FIX: make a complexity function?
        self.write("Complexity {")
        self.write("	value 1.0")
        self.write("	textureQuality 1.")
        self.write("}")
        self.write("Coordinate3 { point [ -"+str(widthInCM/2)+" 0. 0.0, "+str(widthInCM/2)+" 0.0 0.0,")
        self.write("	  -"+str(widthInCM/2)+" 0. -"+str(heightInCM)+", "+str(widthInCM/2)+" 0.0 -"+str(heightInCM)+" ] }")
        self.write("")
        self.write("Texture2 { filename \""+imageFileName+"\" model DECAL }")
        self.write("TextureCoordinate2 { point [")
        self.write("	0.0 1.0,")
        self.write("	1.0 1.0,")
        self.write("	0.0 0.0,")
        self.write("	1.0 0.0,")
        self.write("] }")
        self.write("IndexedFaceSet { coordIndex [ 0, 1, 2, -1,")
        self.write("	      1, 3, 2, -1 ]}")
        self.write("IndexedLineSet { coordIndex [ 0, 1, 3, 2, 0 ] }")
        self.endSeparator("corePhoto "+imageFileName)


    def annotations(self, textlist, scaleText=0.1):
        """
        Write out a line of text at a specified depth

        out - open file handle to write to
        textlist - list of tuples [(001,'junk'),(092,'toy car')]

        >>> [(1,'junk'),(92,'toy car')]
        """
        self.comment("Starting DepthText")
        self.beginSeparator()
        lastDepth=-99999
        minStep = 5
        for item in textlist:
            depth = item[0]
            text  = item[1]
            # Stagger text if it is too close
            if (lastDepth+minStep>depth): offset += 2
            else: offset = 0
            if offset > minStep: offset = 0

            if offset==0:  self.baseColor ([1,0,0.5])
            else: self.baseColor ([1,0.5,0.5])

            lastDepth=depth
            # Item is a tuple or list of the form (92,'toy car')
            self.beginSeparator()
            self.write("Translation { translation 0 "+str(offset)+" -"+str(depth)+" }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            #s = ".1 ";  sss = s + s + s
            s = str(scaleText);  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+text+"\"]}")
            self.endSeparator()
        self.endSeparator("annotations")

    def depthTick(self,maxDepth=200,step=1,width=1):
        """
        Simpler tick maker
        """
        self.write()
        self.comment("DepthTicks")
        self.beginSeparator()

        for depth in range(0,int(maxDepth),step):
            points=[(0,0,-depth),(width,0,-depth)]
            # FIX: LineSet instead!
            self.coordinate3(points)
            self.indexedLineSet([0,1,-1])

        self.endSeparator("depthTicks")

    def depthTicks(self,maxDepth=200,step=10,width=10):
        """
        Make ticks down core
        """
        self.write()
        self.comment("DepthTicks")
        self.beginSeparator()

        for depth in range(0,int(maxDepth),step):
            points=[(0,0,-depth),(width,0,-depth)]
            self.coordinate3(points)
            self.indexedLineSet([0,1,-1])

        for depth in range(0,int(maxDepth),step/2):
            if depth%step == 0: continue  # skip what is already done
            points=[(0,0,-depth),(width/2,0,-depth)]
            self.coordinate3(points)
            self.indexedLineSet([0,1,-1])

        # Quarter ticks
        # FIX: make this work!
#         for depth in range(0,int(maxDepth),step/4.):
#             if depth%step == 0: continue  # skip what is already done
#             if depth%(step/2) == 0: continue  # skip what is already done
#             points=[(0,0,-depth),(width/4,0,-depth)]
#             self.coordinate3(points)
#             self.indexedLineSet([0,1,-1])


        self.endSeparator("depthTicks")

    def depthTextMarkers(self,maxDepth=200,step=10,emphasize=50):
        """
        Create an IV file with text showing the depth down core.

        out - open file handle for writing the iv file
        maxDepth - how far to do down
        step - distance between markings
        emphasize - distance to show in a different color
        """

        self.write()
        self.comment("Starting Depth text markers")
        self.beginSeparator()

        for depth in range(0,int(maxDepth),step):
            if depth%emphasize==0: self.baseColor ([1,1,0])
            self.beginSeparator()
            self.write("Translation { translation 0 0 -"+str(depth)+" }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            s = ".3 ";  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+str(depth)+"\"]}")
            self.endSeparator()

            if depth%emphasize==0: self.baseColor ([1,1,1])
        self.endSeparator("depth text marker")



    def graph(self, data, width=30, title=None):
        """
        Autoscale graph to width

        Arguments:
    
        out - open file handle for writing
        data - list of depth value tuples like [(1, 0.87), (5,-.512), (10, 41.2), (15,-2.3)
        title - String to put at the top

        """
        minValue = 9999999
        maxValue = -9999999
        for item in data:
            if item[1]<minValue: minValue = item[1]
            if item[1]>maxValue: maxValue = item[1]
        
        self.write("")
        self.beginSeparator("graph " + title)
        if title:
            self.beginSeparator()
            self.baseColor(white)
            self.write("Translation { translation "+str(width*.25)+" 0 5 }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            s = ".3 ";  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+title+"\"]}")
            self.endSeparator()

        self.baseColor([1.0,0.1,0.1])
        self.write("Coordinate3 { point [ ")
        # FIX: bug?  where are the commas?
        for item in data:
            depth = item[0]
            value = item[1]
            percent = (value - minValue)/(maxValue - minValue)
            x = percent * width
        self.write("] } # Coord3 ")
    
        self.write("LineSet { vertexProperty VertexProperty { vertex [")
        for item in data:
            depth = item[0]
            value = item[1]
            percent = (value - minValue)/(maxValue - minValue)
            x = percent * width
            self.write("  "+str(x)+" 0 -"+str(depth)+",")
        self.write("] } } # LineSet ")
    
        self.endSeparator("graph " + title)
    
    def graphList(self, dataLists, colorList, width=30, xRange=None, title=None):
        """
        Plot n data streams together

        Arguments:
        dataLists -- n lists of x,y pairs
        colorList -- n lists of colors for how to color the datalists

        e.g.
           from xcore import * # Get the colors
           dataLists = [ [], [], [] ]
           colorList = [ red, green, blue ]
        """
        print "FIX: nothing here!!!"

    def graph3(self, data, width=30, xRange=None, title=None):
        """
        Plot 3 values down core...
        FIX:  Just look at the tuple size to see how many things to plot

        Arguments:

        self - open file handle for writing
        data - list of depth value tuples like [(1, 0.87), (5,-.512), (10, 41.2), (15,-2.3)
        """
        if xRange:
            minValue = xRange[0]
            maxValue = xRange[1]
        else:
            minValue = 9999999
            maxValue = -9999999
            for item in data:
                if item[1]<minValue: minValue = item[1]
                if item[1]>maxValue: maxValue = item[1]
        
        self.beginSeparator("graph3 "+ str(title))

        if title:
            self.beginSeparator()
            self.baseColor(white)
            self.write("Translation { translation "+str(width*.25)+" 0 5 }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            s = ".3 ";  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+title+"\"]}")
            self.endSeparator()

        colors = [(1,0,0),(0,1,0),(0,0,1)]

        for i in range(3):
            self.baseColor([colors[i][0],colors[i][1],colors[i][2]])
            self.write("LineSet { vertexProperty VertexProperty { vertex [")
            for item in data:
                if 4!=len(item):
                    print "Tuple too small:", item
                    sys.exit("Doh!")
                depth = item[0]
                value = item[1+i]
                percent = (value - minValue)/(maxValue - minValue)
                x = percent * width
                self.write("  "+str(x)+" 0 -"+str(depth)+",")
            self.write("] } } # LineSet ")
    
        self.endSeparator()

    def makeKey(self,colorStrList):
        """
        Take color,name tuples and make an iv file key
    
        colorStrList -- List of (color,string) tuples
        	e.g. [('red','this is red'),('blue','this is blue')
        """
    
        colors=X11rgb(maxvalue=1.0)
        #print colors.rgb
    
        #iv = xcore.GenIV(outfile)
        self.header()
        s = 0.3 # Scale factor
        offset = 0 # keeps moving down
        step = 5 # how far between key entries
        sz = 2 # The half hight of the rectange
    
        points = [(-sz,0,+sz), (sz,0,sz), (sz,0,-sz), (-sz,0,-sz)]
        self.coordinate3(points)
        indicesPoly = [2,1,0, -1, 2, 0, 3, -1]
        #print indicesPoly, [2,1,0, -1, 2, 0, 3, -1]
        indicesLines = [0, 1, 2, 3, 0, -1]
    
        for item in colorStrList:
            colorName = item[0]
            string = item[1]
            #colorName = lut[key][1]
            color = colors.rgb[colorName]
            #print colorName, color
            self.comment(string)
            self.beginSeparator()
    
            self.translation(0,0,-offset)
    
            self.baseColor(color)
            self.indexedFaceSet(indicesPoly)
            self.baseColor(colors.rgb['white'])

            self.beginSeparator()
            self.translation(0,-.1,0)
            self.indexedLineSet(indicesLines)
            self.endSeparator()

            if colorName != "black":
                self.baseColor(color)
           
            self.translation(sz+2,0,-sz/2.)
            self.rotationXYZ('X',1.57)
            self.scale(s,s,s)
            #string = lut[key][0]+"  --  "+lut[key][1]
            #print string
            self.text3(string)
    
            self.endSeparator()
            offset += step

        return

    def scatterPlot(self,data,width=50, height=50, border=True,title="Scatter Plot",\
                    minX=None,maxX=None,\
                    minY=None,maxY=None,
                    colors=None,
                    pointSize=3,
                    xlabel=None,
                    ylabel=None):
        """
        Make a scatter plot

        FIX: want a totally separate graphing class!
        FIX: deal with symbols

        data is [(x1,y1),(x2,y2),....]
        """
        if not minX:
            minX = 99999999
            for item in data:
                if item[0]<minX: minX = item[0]
        if not maxX:
            maxX = -99999999
            for item in data:
                if item[0]>maxX: maxX = item[0]

        if not minY:
            minY = 99999999
            for item in data:
                if item[1]<minY: minY = item[1]
        if not maxY:
            maxY = -99999999
            for item in data:
                if item[1]>maxY: maxY = item[1]
        #print (minX,maxX),(minY,maxY)

        assert (minX <= maxX)
        assert (minY <= maxY)

        if title:
            self.beginSeparator()
            self.baseColor(white)
            self.write("Translation { translation "+str(width*.25)+" 0 5 }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            s = ".3 ";  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+title+"\"]}")
            self.endSeparator()

        if xlabel:
            self.beginSeparator()
            self.baseColor(white)
            self.write("Translation { translation "+str(width*.4)+" 0 "+str(-height - 5)+" }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            s = ".3 ";  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+xlabel+"\"]}")
            self.endSeparator()

        if ylabel:
            self.beginSeparator()
            self.baseColor(white)
            self.write("Translation { translation -10 0 "+str(-(height*.5))+" }")
            self.write("RotationXYZ { axis X angle 1.57 }")
            s = ".3 ";  sss = s + s + s
            self.write("Scale { scaleFactor "+sss+" }")
            self.write("Text3{string[\""+ylabel+"\"]}")
            self.endSeparator()

        self.beginSeparator(title)

        if border:
            w = width; h = height
            self.simpleLine([(0,0,0),(w,0,0),(w,0,-h),(0,0,-h),(0,0,0)])

        self.baseColor([1.0,0,0])

        newData=[]
        for point in data:
            x = point[0]
            y = point[1]
            percentX = (x - minX)/(maxX-minX)
            percentY = 1 - (y - minY)/(maxY-minY)
            #percentY = (y - minY)/(maxY-minY)
            x = percentX * width
            y = 0
            z = -1 * percentY * height
            newData.append((x,y,z))
        #print newData
        self.pointSet(newData,colors=colors,size=pointSize)
            
        self.endSeparator(title)

######################################################################
if __name__ == '__main__':
    """ Testing junk.  FIX:  make this a real test harness"""
    iv1 = GenIV("1.iv")
    iv2 = GenIV("2.iv")
    iv1.header()
    iv1.comment("hello")
    iv1.indent()
    iv1.comment("indent1")
    iv1.indent()
    iv1.comment("indent2")
    iv1.unindent(2)
    iv1.comment("hello")

    iv1.beginSeparator()
    iv1.comment("in sep")
    iv1.endSeparator()

    xc1 = GenXCore("3.iv")
    xc1.markRegions([(1,10),(150,200)])
    xc1.corePhoto("foo.png",10,200)
    xc1.includeFile("1.iv")
    xc1.annotations([(1,'junk'),(92,'toy car')])
    xc1.depthTextMarkers()
    xc1.graph([(1, 0.87), (5,-.512), (10, 41.2), (15,-2.3)], title="some data")

    xc1.graph3([(1, 0.87,-.1,9), (5,-.512,0,2), (10, 41.2,10, 20), (15,-2.3,-1,-0.1)], title="some data")
